import { jsx as _jsx } from "react/jsx-runtime";
import { createContext, useCallback, useContext, useEffect, useMemo, useRef, useState, } from 'react';
const PrepareExportContext = createContext(null);
export function usePrepareExport() {
    const context = useContext(PrepareExportContext);
    if (!context) {
        throw new Error('Export prepare context was not found');
    }
    return context;
}
export function useRegisterExportComponents(componentId) {
    const { registerComponent, unRegisterComponent } = usePrepareExport();
    useEffect(() => {
        registerComponent(componentId);
        return () => {
            unRegisterComponent(componentId);
        };
    }, [componentId, registerComponent, unRegisterComponent]);
}
export function useNotifyComponentLoaded(componentId) {
    const { notifyComponentLoaded } = usePrepareExport();
    useEffect(() => {
        notifyComponentLoaded(componentId);
    }, [componentId, notifyComponentLoaded]);
}
export function PrepareExportProvider(props) {
    const { children, timeout = 10000 } = props;
    const [isExporting, setIsExporting] = useState(false);
    const componentRefs = useRef(new Map());
    const registerComponent = useCallback((componentId) => {
        componentRefs.current.set(componentId, false);
    }, []);
    const unRegisterComponent = useCallback((componentId) => {
        componentRefs.current.delete(componentId);
    }, []);
    const notifyComponentLoaded = useCallback((componentId) => {
        componentRefs.current.set(componentId, true);
    }, []);
    const prepareExportStart = useCallback(() => {
        setIsExporting(true);
    }, []);
    const prepareExportEnd = useCallback(() => {
        for (const key of componentRefs.current.keys()) {
            componentRefs.current.set(key, false);
        }
        setIsExporting(false);
    }, []);
    const waitForAllComponentsRendered = useCallback(() => {
        return new Promise((resolve, reject) => {
            const loadPromise = checkComponents(componentRefs.current);
            const timeoutPromise = createTimeoutPromise(timeout);
            Promise.race([loadPromise, timeoutPromise])
                .then(resolve)
                .catch((error) => {
                if (error instanceof Error) {
                    reject(error);
                }
                else {
                    reject(new Error('Unknown error occurred'));
                }
            });
        });
    }, [timeout]);
    const state = useMemo(() => {
        return {
            prepareExportStart,
            prepareExportEnd,
            registerComponent,
            notifyComponentLoaded,
            waitForAllComponentsRendered,
            unRegisterComponent,
            isExporting,
        };
    }, [
        prepareExportStart,
        prepareExportEnd,
        registerComponent,
        notifyComponentLoaded,
        waitForAllComponentsRendered,
        unRegisterComponent,
        isExporting,
    ]);
    return (_jsx(PrepareExportContext.Provider, { value: state, children: children }));
}
const interval = 100;
function checkComponents(componentsStatus) {
    if (componentsStatus.size === 0)
        return Promise.resolve();
    return new Promise((resolve) => {
        const checkInterval = setInterval(() => {
            const allLoaded = Array.from(componentsStatus.values()).every(Boolean);
            if (allLoaded) {
                clearInterval(checkInterval);
                resolve();
            }
        }, interval);
    });
}
function createTimeoutPromise(timeout) {
    return new Promise((_, reject) => {
        setTimeout(() => {
            reject(new Error('Load svg component timeout'));
        }, timeout);
    });
}
export function withExportRegister(Component, componentId) {
    return (props) => {
        const { isExporting } = usePrepareExport();
        useRegisterExportComponents(componentId);
        if (isExporting)
            return null;
        return _jsx(Component, { ...props });
    };
}
export function withExport(Component, componentId) {
    return (props) => {
        const { isExporting } = usePrepareExport();
        useNotifyComponentLoaded(componentId);
        if (!isExporting) {
            return null;
        }
        return _jsx(Component, { ...props });
    };
}
